package com.tztzf.protocol.core;

import com.tztzf.protocol.callback.tztZFBlocktztSSL;
import com.tztzf.protocol.define.tztZFSSLCallbackType;
import com.tztzf.protocol.tztnative.tztNativeBioSSL;
import com.tztzf.tztZFPrivate;

@tztZFPrivate(name = "内部使用类，外层禁止直接调用")
public class tztZFVPNSSL {
    /** 连接类型 8=单向 9=双向 */
    private int connectType;
    /** 回调，用于ssl的读写操作回调处理 */
    private tztZFBlocktztSSL block;
    /** 底层ssl对象 */
    private long bioSSL;
    /** ssl对应使用的socket连接对象 */
    private Object socketFD;
    /*ssl证书*/
    private tztZFSSLCert sslCert;

    /**
     * 证书有效期判断
     *
     * @param cert 证书信息
     * @param type 类型
     * @return true=有效
     */
    public static boolean validityCert(byte[] cert, int type) {
        if (cert == null || cert.length < 1)
            return false;
        return tztNativeBioSSL.ssl_CheckTimeCert(type, cert);
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        ssl_free();
    }

    /**
     * 初始化创建VPNSSL对象
     *
     * @param connectType 连接类型
     * @param block       回调
     */
    public void initWithConnectType(int connectType, tztZFBlocktztSSL block) {
        this.connectType = connectType;
        this.block = block;
        bioSSL = tztNativeBioSSL.createBioSSL(connectType);
        this.socketFD = 0;
    }

    public int ssl_setSSLCert(tztZFSSLCert cert) {
        this.sslCert = cert;
        int nRet = 0;
        if (sslCert != null && sslCert.getCertCA() != null && sslCert.getCertCA().length > 0) {
            nRet = setCaCert(sslCert.getCertCA(), sslCert.getCertCAType());
            if (nRet <= 0)
                return nRet;
        }
        if (sslCert != null && sslCert.getCertEnc() != null && sslCert.getCertEnc().length > 0) {
            nRet = setEncCert(sslCert.getCertEnc(), sslCert.getCertEncKey(), sslCert.getCertEncKeyPass(), sslCert.getCertEncType(), sslCert.getCertType());
            if (nRet <= 0)
                return nRet;
        }
        if (sslCert != null && sslCert.getCertSign() != null && sslCert.getCertSign().length > 0) {
            nRet = setSignCert(sslCert.getCertSign(), sslCert.getCertSignKey(), sslCert.getCertSignKeyPass(), sslCert.getCertSignType());
            if (nRet <= 0)
                return nRet;
        }
        return nRet;
    }

    /**
     * 设置CA证书
     *
     * @param caData 证书内容
     * @param caType 证书类型
     * @return =0，设置证书失败
     */
    public int setCaCert(byte[] caData, int caType) {
        if (bioSSL == 0)
            return -1;
        if (caData == null || caData.length < 1)
            return 0;
        return tztNativeBioSSL.bio_cacert(bioSSL, caData, caType);
    }

    /**
     * 设置签名证书
     *
     * @param sign     签名证书
     * @param signKey  签名key
     * @param signType 签名证书类型
     * @return =0，设置失败
     */
    public int setSignCert(byte[] sign, byte[] signKey, byte[] signKeyPass, int signType) {
        if (bioSSL == 0)
            return -1;
        if (sign == null)
            return 0;
        return tztNativeBioSSL.bio_sign(bioSSL, sign, signKey, signKeyPass, signType);
    }

    /**
     * 设置加密证书
     *
     * @param enc      加密证书
     * @param encKey   加密证书key
     * @param encType  加密证书类型
     * @param certType 证书类型
     * @return =0，设置失败
     */
    public int setEncCert(byte[] enc, byte[] encKey, byte[] encKeyPass, int encType, int certType) {
        if (bioSSL == 0)
            return -1;
        if (enc == null)
            return 0;
        return tztNativeBioSSL.bio_enc(bioSSL, enc, encKey, encKeyPass, encType, certType);
    }

    /**
     * 建立ssl连接
     *
     * @param socket 连接socket
     * @return -1，连接失败
     */
    public int ssl_connect(Object socket) {
        if (bioSSL == 0)
            return -1;
        this.socketFD = socket;
        return tztNativeBioSSL.bio_connect(bioSSL, this);
    }

    /**
     * 处理SSL回调
     *
     * @param type 回调类型
     * @param code 错误码
     * @param data 数据
     * @param len  长度
     * @return 0
     */
    public int socket_block(@tztZFSSLCallbackType int type, int code, byte[] data, int len) {
        if (this.block != null) {
            int ret = this.block.onHandler(socketFD, type, code, data, len);
            return ret;
        }
        return -1;
    }

    public int ssl_init() {
        if (bioSSL == 0)
            return -1;
        return tztNativeBioSSL.bio_init(bioSSL);
    }

    /**
     * ssl读取/接收数据
     *
     * @param data 数据
     * @param len  长度
     * @return 读取的数据长度
     */
    public int ssl_read(byte[] data, int len) {
        if (bioSSL == 0)
            return -1;
        if (len < 1 || data == null || data.length < len) {
            return 0;
        }
        int ret = tztNativeBioSSL.bio_read(bioSSL, data, len);
        return ret;
    }

    /**
     * SSL写/发送数据
     *
     * @param data 数据内容
     * @return 写/发送的数据长度
     */
    public int ssl_write(byte[] data) {
        if (bioSSL == 0)
            return -1;
        if (data == null || data.length < 1)
            return 0;
        int ret = tztNativeBioSSL.bio_write(bioSSL, data);
        return ret;
    }

    /**
     * 关闭ssl连接
     *
     * @return >=0 成功
     */
    public synchronized int ssl_close() {
        if (bioSSL == 0)
            return -1;
        tztNativeBioSSL.bio_close(bioSSL);
        tztNativeBioSSL.freeBioSSL(bioSSL);
        bioSSL = 0;
        return 0;
    }

    public synchronized int ssl_free() {
        if (bioSSL == 0)
            return 0;
        tztNativeBioSSL.bio_close(bioSSL);
        tztNativeBioSSL.freeBioSSL(bioSSL);
        bioSSL = 0;
        return 0;
    }

    ///有效性 0:加密证书 1:签名证书
    public int ssl_verify(int type) {
        if (sslCert == null)
            return -1;
        if (type == 0) {
            return sslCert.verifySign();
        }
        if (type == 1)
            return sslCert.verifyEnc();
        return -1;
    }

    ///有效期 0:加密证书 1:签名证书
    public boolean ssl_validity(int type) {
        if (sslCert == null)
            return false;
        if (type == 0) {
            return sslCert.validityEnc();
        }
        if (type == 1)
            return sslCert.validitySign();
        return false;
    }
}